import json


class EventServer:
    def __init__(self, synapse):
        self._synapse = synapse
        self._channel = synapse.create_channel(process_num=synapse.event_process_num, desc="EventServer")
        self._queue_name = "%s_%s_event" % (self._synapse.sys_name, self._synapse.app_name)

    def _check_and_create_queue(self):
        self._channel.queue_declare(queue=self._queue_name, durable=True, auto_delete=True)
        for k in self._synapse.event_callback.keys():
            self._channel.queue_bind(queue=self._queue_name, exchange=self._synapse.sys_name,
                                     routing_key="event.%s" % k)

    def run(self):
        self._check_and_create_queue()

        def handler(ch, deliver, props, body):
            if deliver != None:
                if self._synapse.debug:
                    self._synapse.log("Event Receive: %s@%s %s" % (props.type, props.reply_to, str(body)))
                key = "%s.%s" % (props.reply_to, props.type)
                if key in self._synapse.event_callback:
                    res = self._synapse.event_callback[key](json.loads(body), props)
                    if res:
                        self._channel.basic_ack(deliver.delivery_tag)
                    else:
                        self._channel.basic_nack(deliver.delivery_tag)
                else:
                    self._channel.basic_nack(deliver.delivery_tag, requeue=False)

        self._channel.basic_consume(consumer_callback=handler, queue=self._queue_name)
        self._channel.start_consuming()



import json
import pika


class EventClient:
    def __init__(self, synapse):
        self._synapse = synapse
        self.ch = synapse.create_channel(desc="EventClient")
        synapse.log("Event Client Ready")

    def send(self, event, params):
        props = pika.BasicProperties(
            app_id=self._synapse.app_id,
            message_id=self._synapse.random_string(),
            reply_to=self._synapse.app_name,
            type=event
        )
        router = "event.%s.%s" % (self._synapse.app_name, event)
        body = json.dumps(params)
        self.ch.basic_publish(exchange=self._synapse.sys_name, routing_key=router, properties=props, body=body)
        if self._synapse.debug:
            self._synapse.log("Event Publish: %s@%s %s" % (event, self._synapse.app_name, body), self._synapse.LogDebug)



import json
import pika


class RpcServer:
    def __init__(self, synapse):
        self._synapse = synapse
        self._channel = synapse.create_channel(process_num=synapse.rpc_process_num, desc="RpcServer")
        self._queue_name = "%s_%s_server" % (self._synapse.sys_name, self._synapse.app_name)
        self._router = "server.%s" % self._synapse.app_name

    def _check_and_create_queue(self):
        self._channel.queue_declare(queue=self._queue_name, durable=True, auto_delete=True)
        self._channel.queue_bind(queue=self._queue_name, exchange=self._synapse.sys_name, routing_key=self._router)

    def run(self):
        self._check_and_create_queue()

        def handler(ch, deliver, props, body):
            if self._synapse.debug:
                self._synapse.log("Rpc Receive: (%s)%s->%s@%s %s" % (
                    props.message_id, props.reply_to, props.type, self._synapse.app_name, str(body)),
                                  self._synapse.LogDebug)
            if props.type in self._synapse.rpc_callback:
                res = self._synapse.rpc_callback[props.type](json.loads(body), props)
            else:
                res = {"rpc_error": "method not found"}
            body = json.dumps(res)
            reply = "client.%s.%s" % (props.reply_to, props.app_id)
            ret_props = pika.BasicProperties(
                app_id=self._synapse.app_id,
                correlation_id=props.message_id,
                message_id=self._synapse.random_string(),
                reply_to=self._synapse.app_name,
                type=props.type
            )
            ch.basic_publish(exchange=self._synapse.sys_name, routing_key=reply, properties=ret_props, body=body)
            if self._synapse.debug:
                self._synapse.log("Rpc Return: (%s)%s@%s->%s %s" % (
                    props.message_id, props.type, self._synapse.app_name, props.reply_to, body), self._synapse.LogDebug)

        self._channel.basic_consume(consumer_callback=handler, queue=self._queue_name, no_ack=True)
        self._channel.start_consuming()



#!/usr/bin/env python
# coding=utf-8

from setuptools import setup, find_packages

setup(
    name='rpc.synpase.astraea',
    version='1.4.3',
    description=(
        'A rpc framework base RabbitMQ'
    ),
    long_description=open('README.md').read(),
    author='xRain',
    author_email='xrain@simcu.com',
    maintainer='xrain0610',
    maintainer_email='xrain@simcu.com',
    license='BSD License',
    packages=find_packages(),
    platforms=["all"],
    url='https://github.com/synapse-rpc',
    classifiers=[
        'Operating System :: OS Independent',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[
        'kombu'
    ],

)



from __future__ import print_function
import time
import pika
import threading
from random import Random
from rpc.synapse.astraea.event_server import EventServer
from rpc.synapse.astraea.event_client import EventClient
from rpc.synapse.astraea.rpc_server import RpcServer
from rpc.synapse.astraea.rpc_client import RpcClient


class Synapse:
    debug = False
    disable_rpc_client = False
    disable_event_client = False
    sys_name = ''
    app_name = ''
    app_id = ''
    mq_host = ''
    mq_port = 5672
    mq_user = ''
    mq_pass = ''
    mq_vhost = '/'
    event_process_num = 20
    rpc_process_num = 20
    rpc_timeout = 3
    event_callback = {}
    rpc_callback = {}

    LogInfo = "Info"
    LogWarn = "Warn"
    LogDebug = "Debug"
    LogError = "Error"

    _rpc_client = None
    _event_client = None

    def serve(self):
        if self.app_name == "" or self.sys_name == "":
            self.log("Must Set app_name and sys_name , system exit .", self.LogError)
            exit(1)
        else:
            self.log("System Name: %s" % self.sys_name)
            self.log("App Name: %s" % self.app_name)
        if self.app_id == "":
            self.app_id = self.random_string()
            self.log("App Id: %s" % self.app_id)
        if self.debug:
            self.log("App Run Mode: Debug", self.LogWarn)
        else:
            self.log("App Run Mode: Production")
        self._check_and_create_exchange()
        if self.event_callback == {}:
            self.log("Event Server Disabled: event_callback not set", self.LogWarn)
        else:
            threading.Thread(target=EventServer(self).run).start()
            for k in self.event_callback:
                self.log("*EVT: %s -> %s" % (k, self.event_callback[k].__name__))
        if self.rpc_callback == {}:
            self.log("Rpc Server Disabled: rpc_callback not set", self.LogWarn)
        else:
            threading.Thread(target=RpcServer(self).run).start()
            for k in self.rpc_callback:
                self.log("*RPC: %s -> %s" % (k, self.rpc_callback[k].__name__))
        if self.disable_event_client:
            self.log("Event Client Disabled: disable_event_client set True", self.LogWarn)
        else:
            self._event_client = EventClient(self)
        if self.disable_rpc_client:
            self.log("Rpc Client Disabled: disable_rpc_client set True", self.LogWarn)
        else:
            self._rpc_client = RpcClient(self)
            threading.Thread(target=self._rpc_client.run).start()

    def send_event(self, event, params):
        if self.disable_event_client:
            self.log("Event Client Disabled!", self.LogError)
        self._event_client.send(event, params)

    def send_rpc(self, app, method, params):
        if self.disable_rpc_client:
            self.log("Rpc Client Disabled!", self.LogError)
            res = {"rpc_error": "rpc client disabled"}
        else:
            res = self._rpc_client.send(app, method, params)
        return res

    def _create_connection(self, desc='unknow'):
        config = pika.ConnectionParameters(
            host=self.mq_host,
            port=self.mq_port,
            virtual_host=self.mq_vhost,
            credentials=pika.PlainCredentials(self.mq_user, self.mq_pass),
            client_properties={"connection_name": "%s_%s_%s_%s" % (self.sys_name, self.app_name, self.app_id, desc)}
        )
        conn = pika.BlockingConnection(parameters=config)
        self.log("%s Connection Created" % desc)
        return conn

    def create_channel(self, process_num=0, desc="unknow", connection=False):
        if not connection:
            connection = self._create_connection(desc=desc)
        channel = connection.channel()
        self.log("%s Channel Created" % desc)
        if process_num > 0:
            channel.basic_qos(prefetch_size=0, prefetch_count=process_num)
            self.log("%s MaxProcessNum: %d" % (desc, process_num))
        return channel

    def _check_and_create_exchange(self):
        connection = self._create_connection(desc="Exchange")
        channel = self.create_channel(desc="Exchange", connection=connection)
        channel.exchange_declare(self.sys_name, 'topic', durable=True, auto_delete=True)
        self.log("Register Exchange Successed.")
        connection.close()
        self.log("Exchange Channel Closed")
        self.log("Exchange Connection Closed")

    def random_string(self, lens=20):
        str = ''
        chars = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
        length = len(chars) - 1
        random = Random()
        for i in range(lens):
            str += chars[random.randint(0, length)]
        return str

    def log(self, desc, level=LogInfo):
        print("[%s][Synapse %s] %s" % (time.strftime("%Y-%m-%d %H:%M:%S"), level, desc))



import json
import pika
import time


class RpcClient:
    _response_cache = {}

    def __init__(self, synapse):
        self._synapse = synapse
        self._channel = synapse.create_channel(desc="RpcClient")
        self._queue_name = "%s_%s_client_%s" % (synapse.sys_name, synapse.app_name, synapse.app_id)
        self._router = "client.%s.%s" % (synapse.app_name, synapse.app_id)
        synapse.log("Rpc Client Ready")

    def _check_and_create_queue(self):
        self._channel.queue_declare(queue=self._queue_name, durable=True, auto_delete=True)
        self._channel.queue_bind(queue=self._queue_name, exchange=self._synapse.sys_name, routing_key=self._router)

    def run(self):
        self._check_and_create_queue()

        def handler(ch, deliver, props, body):
            if self._synapse.debug:
                self._synapse.log("Rpc Response: (%s)%s@%s->%s %s" % (
                    props.correlation_id, props.type, props.reply_to, self._synapse.app_name, str(body)),
                                  self._synapse.LogDebug)
            self._response_cache[props.correlation_id] = json.loads(body)

        self._channel.basic_consume(consumer_callback=handler, queue=self._queue_name, no_ack=True)
        self._channel.start_consuming()

    def send(self, app, method, params):
        props = pika.BasicProperties(
            app_id=self._synapse.app_id,
            message_id=self._synapse.random_string(),
            reply_to=self._synapse.app_name,
            type=method
        )
        router = "server.%s" % app
        body = json.dumps(params)
        self._channel.basic_publish(exchange=self._synapse.sys_name, routing_key=router, properties=props, body=body)
        if self._synapse.debug:
            self._synapse.log("Rpc Request: %s->%s@%s %s" % (self._synapse.app_name, method, app, body),
                              self._synapse.LogDebug)
        ts = int(time.time())
        while True:
            if int(time.time()) - ts > self._synapse.rpc_timeout:
                res = {"rpc_error": "timeout"}
                break
            if props.message_id in self._response_cache:
                res = self._response_cache[props.message_id]
                break
        return res
